/**********
Copyright 1990 Regents of the University of California.  All rights reserved.
Author: 1985 Wayne A. Christopher, U. C. Berkeley CAD Group
**********/

/*
 * Table of available commands.  Note that they're sorted so that the commands
 * that appear in the spiceinit file are at the top.
 */

#include "ngspice.h"
#include "cpdefs.h"
#include "ftedefs.h"
#include "ftehelp.h"
#include "nutctab.h"


/* Bool fields:     stringargs, spiceonly, major */

struct comm nutcp_coms[] = {
    { "let", com_let, FALSE, FALSE, TRUE,
      { 040000, 040000, 040000, 040000 }, E_DEFHMASK, 0, LOTS,
      arg_let,
      "varname = expr : Assign vector variables." } ,
    { "reshape", com_reshape, FALSE, FALSE, TRUE,
      { 040000, 040000, 040000, 040000 }, E_DEFHMASK, 1, LOTS,
      arg_let,
      "vector ... [ shape ] : change the dimensions of a vector." } ,
    { "define", com_define, FALSE, FALSE, TRUE,
      { 010000, 040000, 040000, 040000 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[[func (args)] stuff] : Define a user-definable function." } ,
    { "set", com_set, FALSE, FALSE, TRUE,
      { 020000, 020000, 020000, 020000 }, E_DEFHMASK, 0, LOTS,
      arg_set,
      "[option] [option = value] ... : Set a variable." } ,
    { "alias", com_alias, FALSE, FALSE, FALSE,
      { 02, 04, 04, 04 }, E_ADVANCED, 0, LOTS,
      (int (*)()) NULL,
      "[[word] alias] : Define an alias." } ,
    { "deftype", com_dftype, FALSE, FALSE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 3, LOTS,
      (int (*)()) NULL,
      "spec name pat ... : Redefine vector and plot types.\n" } ,
    { "plot", com_plot, FALSE, FALSE, TRUE,
      { 041000, 041000, 041000, 041000 }, E_BEGINNING | E_HASPLOTS, 1, LOTS,
      arg_plot,
      "expr ... [vs expr] [xl xlo xhi] [yl ylo yhi] : Plot things." },
    { "display", com_display, FALSE, FALSE, TRUE,
      { 040000, 040000, 040000, 040000 }, E_BEGINNING, 0, LOTS,
      arg_display,
      ": Display vector status." } ,
    { "destroy", com_destroy, FALSE, FALSE, FALSE,
      { 0400, 0400, 0400, 0400 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[plotname] ... : Throw away all the data in the plot." } ,
    { "setplot", com_splot, FALSE, FALSE, TRUE,
      { 0400, 0, 0, 0 }, E_DEFHMASK, 0, 1,
      (int (*)()) NULL,
      "[plotname] : Change the current working plot." } ,
    { "setcirc", NULL, FALSE, TRUE, FALSE,
      { 04, 0, 0, 0 }, E_DEFHMASK, 0, 1,
      (int (*)()) NULL,
      "[circuit name] : Change the current circuit." } ,
    { "setscale", com_setscale, FALSE, FALSE, FALSE,
      { 040000, 0, 0, 0 }, E_DEFHMASK, 0, 1,
      (int (*)()) NULL,
      "[vecname] : Change default scale of current working plot." } ,
    { "transpose", com_transpose, FALSE, FALSE, FALSE,
      { 040000, 040000, 040000, 040000 }, E_DEFHMASK, 1, LOTS,
      (int (*)()) NULL,
      "varname ... : Perform matrix transposition on multi-D vectors." } ,
    { "xgraph", com_xgraph, FALSE, FALSE, TRUE,
      { 1, 041000, 041000, 041000 }, E_DEFHMASK, 1, LOTS,
      (int (*)()) NULL,
      "file plotargs : Send plot to Xgraph-11." } ,
    { "hardcopy", com_hardcopy, FALSE, FALSE, TRUE,
      { 1, 041000, 041000, 041000 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "file plotargs : Produce hardcopy plots." } ,
    { "asciiplot", com_asciiplot, FALSE, FALSE, TRUE,
      { 041000, 041000, 041000, 041000 }, E_DEFHMASK, 1, LOTS,
      (int (*)()) NULL,
      "plotargs : Produce ascii plots." } ,
    { "write", com_write, FALSE, FALSE, TRUE,
      { 1, 040000, 040000, 040000 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "file expr ... : Write data to a file." } ,
    { "compose", com_compose, FALSE, FALSE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 2, LOTS,
      (int (*)()) NULL,
      "var parm=val ... : Compose a vector." } ,
    { "unlet", com_unlet, FALSE, FALSE, FALSE,
      { 040000, 040000, 040000, 040000 }, E_DEFHMASK, 1, LOTS,
      (int (*)()) NULL,
      "varname ... : Undefine vectors." } ,
    { "print", com_print, FALSE, FALSE, TRUE,
      { 040000, 040000, 040000, 040000 }, E_BEGINNING, 1, LOTS,
      arg_print,
      "[col] expr ... : Print vector values." } ,
    { "load", com_load, FALSE, FALSE, TRUE,
      { 1, 1, 1, 1 }, E_BEGINNING | E_NOPLOTS, 1, LOTS,
      arg_load,
      "file ... : Load in data." } ,
    { "cross", com_cross, FALSE, FALSE, TRUE,
      { 040000, 0, 040000, 040000 }, E_DEFHMASK, 2, LOTS,
      (int (*)()) NULL,
      "vecname number [ vector ... ] : Make a vector in a strange way." } ,
    { "undefine", com_undefine, FALSE, FALSE, FALSE,
      { 010000, 010000, 010000, 010000 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[func ...] : Undefine a user-definable function." } ,
    { "op", NULL, FALSE, TRUE, TRUE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[.op line args] : Determine the operating point of the circuit." } ,
    { "tran", NULL, FALSE, TRUE, TRUE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[.tran line args] : Do a transient analysis." } ,
    { "ac", NULL, FALSE, TRUE, TRUE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[.ac line args] : Do an ac analysis." } ,
    { "dc", NULL, FALSE, TRUE, TRUE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[.dc line args] : Do a dc analysis." } ,
    { "pz", NULL, FALSE, TRUE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[.pz line args] : Do a pole / zero analysis." } ,
    { "sens", NULL, FALSE, TRUE, TRUE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[.ac line args] : Do a sensitivity analysis." } ,
    { "disto", NULL, FALSE, TRUE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[.disto line args] : Do an distortion analysis." } ,
    { "noise", NULL, FALSE, TRUE, TRUE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[.noise line args] : Do a noise analysis." } ,
    { "listing", NULL, FALSE, TRUE, TRUE,
      { 0100, 0100, 0100, 0100 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[logical] [physical] [deck] : Print the current circuit." } ,
    { "edit", NULL, FALSE, TRUE, TRUE,
      { 1, 0, 0, 0 }, E_DEFHMASK, 0, 1,
      (int (*)()) NULL,
      "[filename] : Edit a spice deck and then load it in." } ,
    { "dump", NULL, FALSE, TRUE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, 0,
      (int (*)()) NULL,
      ": Print a dump of the current circuit." } ,
    { "fourier", com_fourier, FALSE, FALSE, TRUE,
      { 0, 040000, 040000, 040000 }, E_DEFHMASK, 1, LOTS,
      (int (*)()) NULL,
      "fund_freq vector ... : Do a fourier analysis of some data." } ,
    { "show", NULL, FALSE, TRUE, FALSE,
      { 040, 040, 040, 040 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "devspecs ... : parmspecs ... : Print out device parameters." } ,
    { "alter", NULL, FALSE, TRUE, FALSE,
      { 040, 040, 040, 040 }, E_DEFHMASK, 3, LOTS,
      (int (*)()) NULL,
      "devspecs : parmname value : Alter device parameters." } ,
    { "altermod", NULL, FALSE, TRUE, FALSE,
      { 040, 040, 040, 040 }, E_DEFHMASK, 3, LOTS,
      (int (*)()) NULL,
      "devspecs : parmname value : Alter model parameters." } ,
    { "resume", NULL, FALSE, TRUE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, 0,
      (int (*)()) NULL,
      ": Continue after a stop." } ,
    { "state", NULL, FALSE, TRUE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "(unimplemented) : Print the state of the circuit." },
    { "stop", NULL, FALSE, TRUE, FALSE,
      { 04200, 04200, 04200, 04200 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[stop args] : Set a breakpoint." } ,
    { "trace", NULL, FALSE, TRUE, FALSE,
      { 0200, 0200, 0200, 0200 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[all] [node ...] : Trace a node." } ,
    { "save", NULL, FALSE, TRUE, FALSE,
      { 0200, 0200, 0200, 0200 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[all] [node ...] : Save a spice output." } ,
    { "iplot", NULL, FALSE, TRUE, TRUE,
      { 0200, 0200, 0200, 0200 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[all] [node ...] : Incrementally plot a node." } ,
    { "status", NULL, FALSE, TRUE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, 0,
      (int (*)()) NULL,
      ": Print the current breakpoints and traces." } ,
    { "delete", NULL, FALSE, TRUE, FALSE,
      { 020, 020, 020, 020 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[all] [break number ...] : Delete breakpoints and traces." } ,
    { "step", NULL, FALSE, TRUE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, 1,
      (int (*)()) NULL,
      "[number] : Iterate number times, or one." } ,
    { "reset", NULL, FALSE, TRUE, TRUE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, 0,
      (int (*)()) NULL,
      ": Terminate a simulation after a breakpoint (formerly 'end')." } ,
    { "run", NULL, FALSE, TRUE, TRUE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, 1,
      (int (*)()) NULL,
      "[rawfile] : Run the simulation as specified in the input file." } ,
#ifdef notdef
    { "aspice", com_aspice, FALSE, FALSE, FALSE,
      { 1, 1, 1, 1 }, E_DEFHMASK, 1, 2,
      (int (*)()) NULL,
      "file [outfile] : Run a spice job asynchronously." } ,
    { "jobs", com_jobs, FALSE, FALSE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, 0,
      (int (*)()) NULL,
      ": Report on asynchronous spice jobs." } ,
    { "rspice", com_rspice, FALSE, FALSE, FALSE,
      { 1, 1, 1, 1 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[input file] : Run a spice job remotely." } ,
#endif
    { "bug", com_bug, FALSE, FALSE, TRUE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, 0,
      (int (*)()) NULL,
      ": Report a %s bug." } ,
    { "newhelp", com_ahelp, FALSE, FALSE, TRUE,
      { 010, 010, 010, 010 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[command name] ... : help." },
    { "tutorial", com_ghelp, FALSE, FALSE, TRUE,
      { 023010, 023010, 023010, 023010 }, E_BEGINNING, 0, LOTS,
      (int (*)()) NULL,
      "[subject] ... : Hierarchical documentation browser." } ,
    { "help", com_ghelp, FALSE, FALSE, TRUE,
      { 023010, 023010, 023010, 023010 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[subject] ... : Hierarchical documentation browser." } ,
    { "oldhelp", com_help, FALSE, FALSE, TRUE,
      { 010, 010, 010, 010 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[command name] ... : Print help." } ,
    { "quit", com_quit, FALSE, FALSE, TRUE,
      { 0, 0, 0, 0 }, E_BEGINNING, 0, 0,
      (int (*)()) NULL,
      ": Quit %s." } ,
    { "source", nutcom_source, FALSE, FALSE, TRUE,
      { 1, 1, 1, 1 }, E_DEFHMASK, 1, LOTS,
      (int (*)()) NULL,
      "file : Source a %s file." } ,
    { "shift", com_shift, FALSE, FALSE, FALSE,
      { 020000, 0, 0, 0 }, E_DEFHMASK, 0, 2,
      (int (*)()) NULL,
      "[var] [number] : Shift argv or the named list var to the left." } ,
    { "unset", com_unset, FALSE, FALSE, FALSE,
      { 020000, 020000, 020000, 020000 }, E_DEFHMASK, 1, LOTS,
      (int (*)()) NULL,
      "varname ... : Unset a variable." } ,
    { "unalias", com_unalias, FALSE, FALSE, FALSE,
      { 02, 02, 02, 02 }, E_DEFHMASK, 1, LOTS,
      (int (*)()) NULL,
      "word ... : Undefine an alias." } ,
    { "history", com_history, FALSE, FALSE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, 2,
      (int (*)()) NULL,
      "[-r] [number] : Print command history." } ,
    { "echo", com_echo, FALSE, FALSE, FALSE,
      { 1, 1, 1, 1 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[stuff ...] : Print stuff." } ,
    { "shell", com_shell, FALSE, FALSE, TRUE,
      { 1, 1, 1, 1 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[args] : Fork a shell, or execute the command." } ,
    { "rusage", com_rusage, FALSE, FALSE, FALSE,
      { 02000, 02000, 02000, 02000 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[resource ...] : Print current resource usage." } ,
    { "cd", com_chdir, FALSE, FALSE, FALSE,
      { 1, 0, 0, 0 }, E_DEFHMASK, 0, 1,
      (int (*)()) NULL,
      "[directory] : Change working directory." } ,
    { "version", com_version, FALSE, FALSE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "[number] : Print the version number." } ,
    { "diff", com_diff, FALSE, FALSE, FALSE,
      { 0400, 0400, 040000, 040000 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      "plotname plotname [vec ...] : 'diff' two plots." } ,
    { "rehash", com_rehash, FALSE, FALSE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, 0,
      (int (*)()) NULL,
      ": Rebuild the unix command database." } ,
    { "while", NULL, FALSE, FALSE, FALSE,
      { 040000, 040000, 040000, 040000 }, E_DEFHMASK, 1, LOTS,
      (int (*)()) NULL,
      "condition : Execute while the condition is TRUE." } ,
    { "repeat", NULL, FALSE, FALSE, FALSE,
      { 0, 0, 0, 0}, E_DEFHMASK, 0, 1,
      (int (*)()) NULL,
      "[number] : Repeat number times, or forever." } ,
    { "dowhile", NULL, FALSE, FALSE, FALSE,
      { 040000, 040000, 040000, 040000 }, E_DEFHMASK, 1, LOTS,
      (int (*)()) NULL,
      "condition : Execute while the condition is TRUE." } ,
    { "foreach", NULL, FALSE, FALSE, FALSE,
      { 0, 040000, 040000, 040000 }, E_DEFHMASK, 2, LOTS,
      (int (*)()) NULL,
      "variable value ... : Do once for each value." } ,
    { "if", NULL, FALSE, FALSE, FALSE,
      { 040000, 040000, 040000, 040000 }, E_DEFHMASK, 1, LOTS,
      (int (*)()) NULL,
      "condition : Execute if the condition is TRUE." } ,
    { "else", NULL, FALSE, FALSE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, 0,
      (int (*)()) NULL,
      ": Goes with if." } ,
    { "end", NULL, FALSE, FALSE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, 0,
      (int (*)()) NULL,
      ": End a block." } ,
    { "break", NULL, FALSE, FALSE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, 0,
      (int (*)()) NULL,
      ": Break out of a block." } ,
    { "continue", NULL, FALSE, FALSE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, 0,
      (int (*)()) NULL,
      ": Continue a loop." } ,
    { "label", NULL, FALSE, FALSE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 1, 1,
      (int (*)()) NULL,
      "word : Create someplace to go to." } ,
    { "goto", NULL, FALSE, FALSE, FALSE,
      { 0100000, 0, 0, 0 }, E_DEFHMASK, 1, 1,
      (int (*)()) NULL,
      "word : Go to a label." } ,
    { "cdump", com_cdump, FALSE, FALSE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 0, 0,
      (int (*)()) NULL,
      ": Dump the current control structures." } ,
    { "settype", com_stype, FALSE, FALSE, FALSE,
      { 0200000, 040000, 040000, 040000 }, E_DEFHMASK, 2, LOTS,
      (int (*)()) NULL,
      "type vec ... : Change the type of a vector." } ,
    { "strcmp", com_strcmp, FALSE, FALSE, FALSE,
      { 0, 0, 0, 0 }, E_DEFHMASK, 3, 3,
      (int (*)()) NULL,
      "varname s1 s2 : Set $varname to strcmp(s1, s2)." } ,
    { "linearize", NULL, FALSE, TRUE, FALSE,
      { 040000, 040000, 040000, 040000 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      " [ vec ... ] : Convert plot into one with linear scale." } ,

    { 0, NULL, FALSE, FALSE, FALSE, { 0, 0, 0, 0 }, E_DEFHMASK, 0, LOTS,
      (int (*)()) NULL,
      NULL }

} ;

